'' =================================================================================================
''
''   File....... jm_pcd8544_demo.spin 
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen  
''               Copyright (c) 2012 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 18 NOV 2012
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1  = 31                                                     ' programming port
  TX1  = 30
  
  SDA  = 29                                                     ' eeprom
  SCL  = 28

  LED  =  5                                                    ' lcd connections                                                  
  SDIN =  4
  SSCK =  3  
  DC   =  2
  SCE  =  1
  RST  =  0


con

  #0, NO, YES
  

obj

  lcd  : "jm_pcd8544"


pub main | y, level

  lcd.start(RST, LED)                                           ' start lcd w/backlight

  ' simple text
  
  lcd.clear
  lcd.char_xy(0, 2) 
  lcd.str(string(" Nuts & Volts "))
  pause(500)

  lcd.char_xy(0, 3)
  lcd.set_style(lcd#INVERT)                                     ' white pixels on black field
  lcd.str(string(" Nuts & Volts "))  
  pause(2000)

  lcd.clearln(3)                                                ' clear line 3 to black
  pause(1000)

  lcd.set_style(lcd#NORMAL)
  lcd.clearln(3)                                                ' clear line 3 to white
  pause(1000)  


  ' Circle graphic -- not corrected

  lcd.goto_xy(0, 0)                                              
  lcd.put_glyph(@Circle, lcd#DDRAM_SZ)                          ' fill entire screen  
  pause(2000)

  
  ' Circle graphic -- correct aspect ratio

  lcd.goto_xy(0, 0)                                                                                                              
  lcd.put_glyph(@Circle_Fix, lcd#DDRAM_SZ)          
  pause(2000)

  
  ' formatted text output

  show_temps(72)
  pause(2000)


  repeat
    lcd.clear
 
    ' move beanie onto screen from bottom
    ' -- looks like propeller is spinning
     
    repeat y from 5 to 0
      lcd.goto_xy(0, y)
      lcd.put_glyph(@Beanie, lcd#LCD_X_RES * (6-y))
      pause(83)
     
    ' demonstrate backlight control
    
    repeat level from 100 to 1                                  ' dim backlight
      lcd.backlight(level)
      pause(20)
      
    repeat level from 0 to 99                                   ' brighten backlight
      lcd.backlight(level)
      pause(20)

    ' demonstrate display control

    lcd.display_ctrl(lcd#DISP_INVERT)                           ' invert display
    pause(500)

    lcd.display_ctrl(lcd#DISP_ALL)                              ' fill all pixels
    pause(500)

    lcd.display_ctrl(lcd#DISP_BLANK)                            ' clear all pixels
    pause(500)    

    lcd.display_ctrl(lcd#DISP_NORMAL)                           ' back to normal
    pause(500)     


pub show_temps(tf) | tc

  tf *= 10                                                      ' convert to 10ths
  tc := (tf - 32_0) * 5 / 9                                     ' convert to Celsius

  lcd.clear                                                     ' clear the display
  lcd.write($7E, lcd#LCD_DATA)                                  ' create end cap   
  lcd.wr_count($FF, 82)                                         ' reverse top line
  lcd.write($7E, lcd#LCD_DATA)                                  ' create end cap

  lcd.set_style(lcd#INVERT)                                     ' inverted text
  lcd.center(string("Temperature"), 0)                          ' center on line
  lcd.set_style(lcd#NORMAL)     

  lcd.char_xy(0, 2)
  lcd.rjdec(tf/10, 7, " ")                                      ' show whole number
  lcd.putc(".") 
  lcd.hex(tf//10, 1)                                            ' hex faster for 1 digit  
  lcd.putc(" ")                               
  lcd.put_glyph(@DegSym, 6)  
  lcd.putc("F")      

  lcd.char_xy(0, 3)
  lcd.rjdec(tc/10, 7, " ") 
  lcd.putc(".") 
  lcd.hex(tc//10, 1)
  lcd.putc(" ")
  lcd.put_glyph(@DegSym, 6)  
  lcd.putc("C")  


con

  { --------------- }
  {                 }
  {   B A S I C S   }
  {                 }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

con

  { --------------------------------- }
  {                                   }
  {   L C D   D E F I N I T I O N S   }
  {                                   } 
  { --------------------------------- }

  
dat

' Custom characters (use put_glyph)

DegSym          byte    $06, $09, $09, $06, $00, $00

DegSym2         byte    %00000110
                byte    %00001001
                byte    %00001001
                byte    %00000110
                byte    %00000000
                byte    %00000000

Diamond         byte    $08, $1C, $3E, $1C, $08, $00 


' Files generated by LCD Assistant          
' http://en.radzio.dxp.pl/bitmap_converter 
' -- converted to Propeller format by JonnyMac

Circle          byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $80, $C0, $C0
                byte    $E0, $E0, $F0, $F0, $F0, $F8, $F8, $F8, $F8, $F8, $F8, $F8, $F8, $F8, $F8, $F0
                byte    $F0, $F0, $E0, $E0, $C0, $C0, $80, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $C0, $F0, $F8, $FC, $FE
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FE, $FC, $F8, $F0
                byte    $C0, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $F8, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $F8, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $1F, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $1F, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $03, $0F, $1F, $3F, $7F, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $7F, $3F, $1F, $0F, $03, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $01, $03, $03, $07, $07, $0F, $0F, $0F, $1F, $1F, $1F, $1F, $1F, $1F, $1F
                byte    $1F, $1F, $1F, $0F, $0F, $0F, $07, $07, $03, $03, $01, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00


' This circle has correct aspect ratio
' -- use graphics program to scale X width by 120%
                
Circle_Fix      byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $80, $C0, $C0, $E0, $E0
                byte   $E0, $F0, $F0, $F0, $F0, $F8, $F8, $F8, $F8, $F8, $F8, $F8, $F8, $F8, $F8, $F8
                byte   $F0, $F0, $F0, $F0, $E0, $E0, $E0, $C0, $C0, $80, $00, $00, $00, $00, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $80, $E0, $F0, $F8, $FC, $FE, $FF, $FF, $FF
                byte   $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte   $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte   $FE, $FC, $F8, $F0, $E0, $80, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $F8, $FE, $FF, $FF, $FF, $FF, $FF
                byte   $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte   $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte   $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FE, $F0, $00, $00, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $1F, $7F, $FF
                byte   $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte   $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte   $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $7F, $0F
                byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte   $00, $00, $00, $01, $07, $0F, $1F, $3F, $7F, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte   $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte   $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $7F, $3F, $1F, $0F
                byte   $07, $01, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $01
                byte   $03, $03, $07, $07, $07, $0F, $0F, $0F, $0F, $1F, $1F, $1F, $1F, $1F, $1F, $1F
                byte   $1F, $1F, $1F, $1F, $0F, $0F, $0F, $0F, $07, $07, $07, $03, $03, $01, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte   $00, $00, $00, $00, $00, $00, $00, $00


Beanie          byte    $00, $00, $00, $00, $00, $00, $00, $00, $F0, $F0, $F8, $F8, $F8, $F8, $F8, $F8
                byte    $F8, $F8, $F0, $F0, $F0, $F0, $F0, $F0, $F0, $E0, $E0, $E0, $E0, $E0, $E0, $E0
                byte    $E0, $E0, $60, $60, $60, $60, $60, $60, $70, $F0, $70, $20, $30, $30, $30, $30
                byte    $30, $30, $38, $38, $38, $38, $38, $3C, $3C, $7C, $7C, $7C, $7C, $7C, $7C, $7C
                byte    $FE, $FE, $FE, $FE, $FE, $FE, $FC, $FC, $FC, $78, $78, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $01, $01, $01
                byte    $01, $03, $03, $03, $03, $03, $03, $01, $01, $01, $01, $01, $01, $01, $01, $01
                byte    $00, $80, $80, $80, $C0, $C0, $40, $40, $60, $20, $A0, $F8, $FF, $FF, $FF, $F8
                byte    $A0, $20, $60, $40, $40, $C0, $C0, $80, $80, $80, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $80, $C0, $E0, $F0, $F8, $F8
                byte    $FC, $7E, $1E, $0F, $07, $03, $01, $01, $00, $00, $00, $E0, $FC, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FC, $E0, $00, $00, $00, $01, $01, $03, $07, $0F
                byte    $1E, $7E, $FC, $F8, $F8, $F0, $E0, $C0, $80, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $C0, $F0, $F8, $FE, $FF, $FF, $FF
                byte    $FF, $7F, $0F, $03, $00, $00, $00, $00, $00, $00, $00, $00, $00, $80, $FE, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FE, $80, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $03, $0F, $7F, $FF, $FF, $FF, $FF, $FE, $F8
                byte    $F0, $C0, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $C0, $FE, $FF, $FF, $FF
                byte    $FF, $7F, $7F, $7F, $77, $70, $30, $30, $30, $30, $30, $30, $38, $38, $38, $38
                byte    $18, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $1F
                byte    $1F, $1F, $18, $38, $38, $38, $38, $30, $30, $30, $30, $30, $30, $70, $77, $7F
                byte    $7F, $7F, $FF, $FF, $FF, $FF, $FE, $C0, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $01
                byte    $07, $07, $0E, $0C, $0C, $0C, $1C, $18, $18, $18, $18, $18, $38, $38, $30, $30
                byte    $30, $30, $30, $70, $60, $60, $60, $60, $60, $60, $60, $60, $60, $60, $60, $60
                byte    $60, $60, $60, $60, $60, $60, $60, $70, $30, $30, $30, $30, $30, $38, $38, $18
                byte    $18, $18, $18, $18, $1C, $0C, $0C, $0C, $0E, $07, $07, $01, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00 


dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}